@@ -1,14 +1,16 @@
-# Copyright (c) 2013-2015 University Corporation for Atmospheric Research/Unidata.
-# Distributed under the terms of the MIT License.
-# SPDX-License-Identifier: MIT
+# Copyright (c) 2013-2015 Siphon Contributors.
+# Distributed under the terms of the BSD 3-Clause License.
+# SPDX-License-Identifier: BSD-3-Clause
 """Read upper air data from the Wyoming archives."""
 
+from datetime import datetime
 from io import StringIO
 import warnings
 
 from bs4 import BeautifulSoup
 import numpy as np
 import pandas as pd
+
 from .._tools import get_wind_components
 from ..http_util import HTTPEndPoint
 
@@ -44,10 +46,10 @@ class WyomingUpperAir(HTTPEndPoint):
 
         """
         endpoint = cls()
-        df = endpoint._get_data(time, site_id, **kwargs)
+        df = endpoint._get_data(time, site_id)
         return df
 
-    def _get_data(self, time, site_id, region='naconf'):
+    def _get_data(self, time, site_id):
         r"""Download and parse upper air observations from an online archive.
 
         Parameters
@@ -59,17 +61,16 @@ class WyomingUpperAir(HTTPEndPoint):
             The three letter ICAO identifier of the station for which data should be
             downloaded.
 
-        region
-            Region to request data from
-
         Returns
         -------
             :class:`pandas.DataFrame` containing the data
 
         """
-        raw_data = self._get_data_raw(time, site_id, region)
+        raw_data = self._get_data_raw(time, site_id)
+        soup = BeautifulSoup(raw_data, 'html.parser')
+        tabular_data = StringIO(soup.find_all('pre')[0].contents[0])
         col_names = ['pressure', 'height', 'temperature', 'dewpoint', 'direction', 'speed']
-        df = pd.read_fwf(raw_data, skiprows=5, usecols=[0, 1, 2, 3, 6, 7], names=col_names)
+        df = pd.read_fwf(tabular_data, skiprows=5, usecols=[0, 1, 2, 3, 6, 7], names=col_names)
         df['u_wind'], df['v_wind'] = get_wind_components(df['speed'],
                                                          np.deg2rad(df['direction']))
 
@@ -77,6 +78,23 @@ class WyomingUpperAir(HTTPEndPoint):
         df = df.dropna(subset=('temperature', 'dewpoint', 'direction', 'speed',
                                'u_wind', 'v_wind'), how='all').reset_index(drop=True)
 
+        # Parse metadata
+        meta_data = soup.find_all('pre')[1].contents[0]
+        lines = meta_data.splitlines()
+        station = lines[1].split(':')[1].strip()
+        station_number = int(lines[2].split(':')[1].strip())
+        sounding_time = datetime.strptime(lines[3].split(':')[1].strip(), '%y%m%d/%H%M')
+        latitude = float(lines[4].split(':')[1].strip())
+        longitude = float(lines[5].split(':')[1].strip())
+        elevation = float(lines[6].split(':')[1].strip())
+
+        df['station'] = station
+        df['station_number'] = station_number
+        df['time'] = sounding_time
+        df['latitude'] = latitude
+        df['longitude'] = longitude
+        df['elevation'] = elevation
+
         # Add unit dictionary
         df.units = {'pressure': 'hPa',
                     'height': 'meter',
@@ -85,10 +103,16 @@ class WyomingUpperAir(HTTPEndPoint):
                     'direction': 'degrees',
                     'speed': 'knot',
                     'u_wind': 'knot',
-                    'v_wind': 'knot'}
+                    'v_wind': 'knot',
+                    'station': None,
+                    'station_number': None,
+                    'time': None,
+                    'latitude': 'degrees',
+                    'longitude': 'degrees',
+                    'elevation': 'meter'}
         return df
 
-    def _get_data_raw(self, time, site_id, region='naconf'):
+    def _get_data_raw(self, time, site_id):
         """Download data from the University of Wyoming's upper air archive.
 
         Parameters
@@ -97,25 +121,21 @@ class WyomingUpperAir(HTTPEndPoint):
             Date and time for which data should be downloaded
         site_id : str
             Site id for which data should be downloaded
-        region : str
-            The region in which the station resides. Defaults to `naconf`.
 
         Returns
         -------
-        a file-like object from which to read the data
+        text of the server response
 
         """
-        path = ('?region={region}&TYPE=TEXT%3ALIST'
+        path = ('?region=naconf&TYPE=TEXT%3ALIST'
                 '&YEAR={time:%Y}&MONTH={time:%m}&FROM={time:%d%H}&TO={time:%d%H}'
-                '&STNM={stid}').format(region=region, time=time, stid=site_id)
+                '&STNM={stid}').format(time=time, stid=site_id)
 
         resp = self.get_path(path)
         # See if the return is valid, but has no data
         if resp.text.find('Can\'t') != -1:
             raise ValueError(
-                'No data available for {time:%Y-%m-%d %HZ} from region {region} '
-                'for station {stid}.'.format(time=time, region=region,
-                                             stid=site_id))
+                'No data available for {time:%Y-%m-%d %HZ} '
+                'for station {stid}.'.format(time=time, stid=site_id))
 
-        soup = BeautifulSoup(resp.text, 'html.parser')
-        return StringIO(soup.find_all('pre')[0].contents[0])
+        return resp.text
